/*
** Module   : CommonWidget.java
** Abstract : Define all widget-related APIs available via a widget handle
**            instance.
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 CA  20130112 Created initial version.
** 002 CS  20130204    Added conversion support for socket and server-socket 
**                     - moved sensitive related methods from 
**                     CommonWidget in the Sensitive interface.
** 003 CA  20130221 Refactored usage of attributes defined by the CommonHandleChain, CommonField
**                  and Sizeable interfaces.
** 004 EVL 20130222 Adding UI related attributes and methods.
** 005 OM  20130304 Refactored isValid and isUnknown of WrappedResource to valid and unknown.
** 006 CA  20130323 The PARENT attribute must be a CommonWidget, not Object.
**                  Added moveAfterTAb(handle) and moveBeforeTabItem(handle) APIs.
** 007 CA  20130323 The PARENT and FRAME attributes need to be set/get via handles.
** 008 VMN 20130401 Corrected fetchSelectedRow(), int is changed to NumberType and long.
*/

package com.goldencode.p2j.ui;

import com.goldencode.p2j.persist.P2JQuery;
import com.goldencode.p2j.util.*;

/**
 * Provide access to all widget-related APIs which can be accessed via a 
 * handle instance.
 */
public interface CommonWidget
extends CommonHandleChain,
        CommonField,
        Sizeable
{
   /**
    * Blocks attempts to access this widget method.
    * 
    * @param   itemList
    *          Not used.
    * 
    * @return  nothing.
    */
   public logical addFirst(character itemList);

   /**
    * Blocks attempts to access this widget method.
    * 
    * @param   label
    *          Not used.
    * @param   value
    *          Not used.
    * 
    * @return  nothing.
    */
   public logical addFirst(character label, character value);

   /**
    * Blocks attempts to access this widget method.
    * 
    * @param   itemList
    *          Not used.
    * 
    * @return  nothing.
    */
   public logical addFirst(String itemList);

   /**
    * Blocks attempts to access this widget method.
    * 
    * @param   label
    *          Not used.
    * @param   value
    *          Not used.
    * 
    * @return  nothing.
    */
   public logical addFirst(String label, String value);

   /**
    * Applies an event to this widget.
    * <p>
    * This method corresponds to the Progress statement: APPLY widget
    * 
    * @param    event
    *           The name of the event.
    */
   public void apply(character event);

   /**
    * Applies an event to this widget.
    * <p>
    * This method corresponds to the Progress statement: APPLY widget
    * 
    * @param    eventCode
    *           The event code.
    */
   public void apply(double eventCode);

   /**
    * Applies an event to this widget.
    * <p>
    * This method corresponds to the Progress statement: APPLY widget
    * 
    * @param    eventCode
    *           The event code.
    */
   public void apply(integer eventCode);

   /**
    * Applies an event to this widget.
    * <p>
    * This method corresponds to the Progress statement: APPLY widget
    * 
    * @param    event
    *           The name of the event.
    */
   public void apply(String event);

   /**
    * Get this widget wrapped in a handle.
    * 
    * @return  handle to this frame.
    */
   public handle asWidgetHandle();

   /**
    * COLOR DISPLAY
    * 
    * @param   cs
    *          Color specification.
    */
   public void colorDisplay(ColorSpec cs);

   /**
    * COLOR PROMPT
    * 
    * @param   cs
    *          Color specification.
    */
   public void colorPrompt(ColorSpec cs);

   /**
    * Blocks attempts to access this widget method.
    * 
    * @return  nothing.
    */
   public logical deselectRows();

   /**
    * Disables this widget.
    * <p>
    * This method corresponds to the Progress statement: DISABLE widget
    */
   public void disable();

   /**
    * Enables this widget.
    * <p>
    * This method corresponds to the Progress statement: ENABLE widget
    */
   public void enable();

   /**
    * Blocks attempts to access this widget method.
    * 
    * @param   index
    *          Not used.
    * 
    * @return  nothing.
    */
   public character entry(integer index);

   /**
    * Blocks attempts to access this widget method.
    * 
    * @param   n
    *          Not used.
    * 
    * @return  nothing.
    */
   public logical fetchSelectedRow(NumberType n);

   /**
    * Blocks attempts to access this widget method.
    *
    * @param   n
    *          Not used.
    *
    * @return  nothing.
    */
   public logical fetchSelectedRow(long n);

   /**
    * Blocks attempts to access this widget method.
    * 
    * @param   index
    *          Not used.
    * 
    * @return  nothing.
    */
   public BrowseColumnWidget getBrowseColumn(double index);

   /**
    * Blocks attempts to access this widget method.
    * 
    * @param   index
    *          Not used.
    * 
    * @return  nothing.
    */
   public BrowseColumnWidget getBrowseColumn(NumberType index);

   /**
    * Blocks attempts to access this attribute.
    * 
    * @return  nothing.
    */
   public decimal getColumn();

   /**
    * Blocks attempts to access this attribute.
    * 
    * @return  nothing.
    */
   public integer getDcolor();

   /**
    * Accesses the first widget in the list of contained widgets.
    *
    * @return   The first widget or <code>null</code> if no widgets are
    *           contained.
    */
   public GenericWidget getFirstChild();

   /**
    * Gets the containing frame instance.
    * 
    * @return   The frame that contains this widget.
    */
   public GenericFrame getFrame();

   /**
    * Gets the FRAME attribute, which is the containing frame instance, as a handle.
    * 
    * @return   The frame that contains this widget.
    */
   public handle getFrameHandle();

   /**
    * Gets the HEIGHT attribute.
    *
    * @return   The current value of the HEIGHT attribute.
    */
   public decimal getHeight();

   /**
    * Get the HELP writable attribute.
    * 
    * @return  current value of HELP attribute
    */
   public character getHelp();

   /**
    * Gets the numeric ID of this widget (this is used as an index into the
    * screen-buffer among other things).
    * 
    * @return   The numeric ID of this widget.
    */
   public int getId();

   /**
    * Blocks attempts to access this attribute.
    * 
    * @return  nothing.
    */
   public integer getInnerChars();

   /**
    * Blocks attempts to access this attribute.
    * 
    * @return  nothing.
    */
   public integer getInnerLines();

   /**
    * Get the label text for this widget.
    *
    * @return   The label text.
    */
   public String getLabelStr();

   /**
    * Accesses the last widget in the list of contained widgets.
    *
    * @return   The last widget or <code>null</code> if no widgets are
    *           contained.
    */
   public GenericWidget getLastChild();

   /**
    * Blocks attempts to access this attribute.
    * 
    * @return  nothing.
    */
   public character getListItemPairs();

   /**
    * Blocks attempts to access this attribute.
    * 
    * @return  nothing.
    */
   public character getListItems();

   /**
    * Blocks attempts to access this attribute.
    * 
    * @return  nothing.
    */
   public decimal getMaxHeightChars();

   /**
    * Blocks attempts to access this attribute.
    * 
    * @return  nothing.
    */
   public decimal getMaxWidthChars();

   /**
    * Blocks attempts to access this attribute.
    * 
    * @return  nothing.
    */
   public GenericWidget getNextTabItem();

   /**
    * Blocks attempts to access this widget method.
    * 
    * @return  nothing.
    */
   public integer getNumColumns();

   /**
    * Blocks attempts to access this attribute.
    * 
    * @return  nothing.
    */
   public integer getNumSelectedRows();

   /**
    * Blocks attempts to access this attribute.
    * 
    * @return  nothing.
    */
   public CommonWidget getParent();

   /**
    * Gets the PARENT attribute, as a handle.
    * 
    * @return  the PARENT attribute.
    */
   public handle getParentHandle();

   /**
    * Blocks attempts to access this attribute.
    * 
    * @return  nothing.
    */
   public integer getPfColor();

   /**
    * Blocks attempts to access this attribute.
    * 
    * @return  nothing.
    */
   public P2JQuery getQuery();

   /**
    * Blocks attempts to access this attribute.
    * 
    * @return  nothing.
    */
   public decimal getRow();

   /**
    * Obtain the current value in the screen buffer of the backing data for
    * this widget and return it as a <code>character</code> type. If
    * there is no value in the screen buffer (if it is <code>null</code>
    * which is the representation for an uninitialized value), then return
    * the empty string.
    *
    * @return   The value as a <code>character</code> type or the empty
    *           string if the screen buffer value is uninitialized.
    */
   public character getScreenValue();

   /**
    * Queries the TABLE attribute of this widget.
    *
    * @return The name of the database table containing the field associated
    *         with the widget.
    */
   public character getTable();

   /**
    * Blocks attempts to access this attribute.
    * 
    * @return  nothing.
    */
   public character getTitle();

   /**
    * Queries the TOOLTIP attribute of this widget.
    *
    * @return A help text message for a text field or text variable.
    */
   public character getTooltip();

   /**
    * Gets the WIDTH attribute.
    *
    * @return   The current value of the WIDTH attribute.
    */
   public decimal getWidth();

   /**
    * Blocks attempts to access this attribute.
    * 
    * @return  nothing.
    */
   public integer getX();

   /**
    * Blocks attempts to access this attribute.
    * 
    * @return  nothing.
    */
   public integer getY();

   /**
    * Hides this widget.
    * <p>
    * This method corresponds to the Progress statement: HIDE widget
    */
   public void hide();

   /**
    * Hides this widget.
    * <p>
    * This method corresponds to the Progress statement: HIDE widget
    * 
    * @param   noPause
    *          <code>true</code> causes omission of any pauses which would
    *          be necessary otherwise.
    */
   public void hide(boolean noPause);

   /**
    * Checks whether the widget is changed during the most recent operation.
    * 
    * @return   <code>true</code> if this widget has changed its value.
    */
   public logical isEntered();

   /**
    * Gets the HIDDEN writable attribute.
    *
    * @return   The current value of the HIDDEN attribute.
    */
   public logical isHidden();

   /**
    * Blocks attempts to access this attribute.
    * 
    * @return  nothing.
    */
   public logical isKeepFrameZOrder();

   /**
    * Blocks attempts to access this attribute.
    * 
    * @return  nothing.
    */
   public logical isMultiple();

   /**
    * Checks whether the widget is not changed during the most recent
    * operation.
    * 
    * @return   <code>true</code> if this widget has not changed its value.
    */
   public logical isNotEntered();

   /**
    * Checks whether the widget is write-protected.
    * 
    * @return   <code>true</code> if the widget is write-protected.
    */
   public logical isReadOnly();

   /**
    * Blocks attempts to access this attribute.
    * 
    * @return  nothing.
    */
   public logical isScrollable();

   /**
    * Blocks attempts to access this attribute.
    * 
    * @return  nothing.
    */
   public logical isSelected();

   /**
    * Blocks attempts to access this widget method.
    * 
    * @param   item
    *          Not used.
    * 
    * @return  nothing.
    */
   public logical isSelected(character item);

   /**
    * Blocks attempts to access this widget method.
    * 
    * @param   index
    *          Not used.
    * 
    * @return  nothing.
    */
   public logical isSelected(integer index);

   /**
    * Reports if this object is unknown.<p>
    * This implementation always returns <code>false</code>.
    *
    * @return   <code>false</code>.
    */
   public boolean unknown();

   /**
    * Reports if this widget object is valid for use.
    * <p>
    * The validity of the widget object is determined solely by the scope of
    * the containing frame.
    * 
    * @return   <code>true</code> if we are valid (can be used).
    */
   public boolean valid();

   /**
    * Checks if the widget is validateable.
    * 
    * @return   <code>true</code> if the widget is validateable.
    */
   public boolean isValidatable();

   /**
    * Get the VISIBLE writable attribute.
    *
    * @return   The current value of VISIBLE attribute.
    */
   public logical isVisible();

   /**
    * Blocks attempts to access this widget method.
    * 
    * @param   text
    *          Not used.
    * 
    * @return  nothing.
    */
   public integer lookup(character text);

   /**
    * Blocks attempts to access this widget method.
    * 
    * @param   text
    *          Not used.
    * 
    * @return  nothing.
    */
   public integer lookup(String text);

   /**
    * This method implements MOVE-AFTER-TAB function.
    *
    * @param   widget
    *          Widget reference.
    *
    * @return  <code>true</code> if operation was successful.
    */
   public logical moveAfterTab(GenericWidget widget);

   /**
    * This method implements MOVE-AFTER-TAB function.
    *
    * @param   widget
    *          The handle of the widget after whose tab position this will be moved.
    *
    * @return  <code>true</code> if operation was successful.
    */
   public logical moveAfterTab(handle widget);

   /**
    * This method implements MOVE-TO-BOTTOM function.
    *
    * @return  <code>true</code> if operation was successful.
    */
   public logical moveToBottom();

   /**
    * This method implements MOVE-TO-TOP function.
    *
    * @return  <code>true</code> if operation was successful.
    */
   public logical moveToTop();

   /**
    * Blocks attempts to access this widget method.
    * 
    * @return  nothing.
    */
   public logical refresh();

   /**
    * Restore configured colors.
    */
   public void restoreColors();

   /**
    * Save configured colors.
    */
   public void saveColors();

   /**
    * Blocks attempts to access this widget method.
    * 
    * @return  nothing.
    */
   public logical selectAll();

   /**
    * Blocks attempts to access this widget method.
    * 
    * @return  nothing.
    */
   public logical selectFocusedRow();

   /**
    * Sets the AUTO-RETURN option.
    * 
    * @param    autoReturn
    *           <code>true</code> if the option is set to ON.
    */
   public void setAutoReturn(boolean autoReturn);

   /**
    * Sets the BLANK option.
    * 
    * @param    blank
    *           <code>true</code> if the option is set to ON.
    */
   public void setBlank(boolean blank);

   /**
    * Blocks attempts to access this attribute.
    * 
    * @param   column
    *          Not used.
    */
   public void setColumn(double column);

   /**
    * Blocks attempts to access this attribute.
    * 
    * @param   column
    *          Not used.
    */
   public void setColumn(NumberType column);

   /**
    * Sets the display color.
    * 
    * @param    displayColor
    *           The display color.
    */
   public void setDcolor(Color displayColor);

   /**
    * Sets the display color.
    *
    * @param    param
    *           The display color.
    */
   public void setDcolor(NumberType param);

   /**
    * Sets the DEBLANK option.
    * 
    * @param    deblank
    *           <code>true</code> if the option is set to ON.
    */
   public void setDeblank(boolean deblank);

   /**
    * Sets the DISABLE-AUTO-ZAP option.
    * 
    * @param    disableAutoZap
    *           <code>true</code> if the option is set to ON.
    */
   public void setDisableAutoZap(boolean disableAutoZap);

   /**
    * Sets the format specification.
    * 
    * @param    format
    *           The format specification.
    */
   public void setFormat(String format);

   /**
    * Sets the containing frame instance.
    * 
    * @param    frame
    *           <code>GenericFrame</code> instance that contains this widget
    */
   public void setFrame(FrameWidget frame);

   /**
    * Sets the FRAME attribute, containing frame instance.
    * 
    * @param    h
    *           a handle containing the <code>GenericFrame</code> instance that contains
    *           this widget
    */
   public void setFrameHandle(handle h);

   /**
    * Sets the containing frame instance.
    * 
    * @param    frame
    *           <code>GenericFrame</code> instance that contains this widget
    */
   public void setFrame(GenericFrame frame);

   /**
    * Sets the containing frame instance.
    * 
    * @param    hFrame
    *           handle which is expected to contain a <code>FrameWidget</code>
    *           instance
    */
   public void setFrame(handle hFrame);

   /**
    * Establishes a relationship between the container and the elements in
    * a given widget group.
    *
    * @param    group
    *           The container of the widget.
    */
   public void setGroup(Group group);

   /**
    * Mark widget as a header.
    * 
    * @param    header
    *           New value of the flag.
    */
   public void setHeader(boolean header);

   /**
    * Sets the help text.
    * 
    * @param    help
    *           The new help text.
    */
   public void setHelp(character help);

   /**
    * Sets the help text.
    * 
    * @param    help
    *           The new help text.
    */
   public void setHelp(String help);

   /**
    * Sets the HIDDEN writable attribute.
    *
    * @param    hidden
    *           The new value for the HIDDEN attribute.
    */
   public void setHidden(boolean hidden);

   /**
    * Sets the HIDDEN writable attribute.
    *
    * @param    hidden
    *           The new value for the HIDDEN attribute.
    */
   public void setHidden(logical hidden);

   /**
    * Sets the numeric ID of this widget (this is used as an index into the
    * screen-buffer among other things).
    * 
    * @param    id
    *           The numeric ID of this widget.
    */
   public void setId(int id);

   /**
    * Blocks attempts to access this attribute.
    * 
    * @param   ic
    *          Not used.
    */
   public void setInnerChars(integer ic);

   /**
    * Blocks attempts to access this attribute.
    * 
    * @param   il
    *          Not used.
    */
   public void setInnerLines(integer il);

   /**
    * Blocks attempts to access this attribute.
    * 
    * @param   k
    *          Not used.
    */
   public void setKeepFrameZOrder(boolean k);

   /**
    * Blocks attempts to access this attribute.
    * 
    * @param   k
    *          Not used.
    */
   public void setKeepFrameZOrder(logical k);

   /**
    * Blocks attempts to access this attribute.
   
    * @param   l
    *          Not used.
    */
   public void setListItemPairs(character l);

   /**
    * Blocks attempts to access this attribute.
    * 
    * @param   l
    *          Not used.
    */
   public void setListItemPairs(String l);

   /**
    * Blocks attempts to access this attribute.
    * 
    * @param   l
    *          Not used.
    */
   public void setListItems(character l);

   /**
    * Blocks attempts to access this attribute.
    * 
    * @param   l
    *          Not used.
    */
   public void setListItems(String l);

   /**
    * Blocks attempts to access this attribute.
    * 
    * @param   m
    *          Not used.
    */
   public void setMaxHeightChars(decimal m);

   /**
    * Blocks attempts to access this attribute.
    * 
    * @param   m
    *          Not used.
    */
   public void setMaxWidthChars(decimal m);

   /**
    * Blocks attempts to access this attribute.
    * 
    * @param   multiple
    *          Not used.
    */
   public void setMultiple(boolean multiple);

   /**
    * Blocks attempts to access this attribute.
    * 
    * @param   multiple
    *          Not used.
    */
   public void setMultiple(logical multiple);

   /**
    * Blocks attempts to access this attribute.
    * 
    * @param   n
    *          Not used.
    */
   public void setNextTabItem(GenericWidget n);

   /**
    * Blocks attempts to access this attribute.
    * 
    * @param   r
    *          Not used.
    */
   public void setNumSelectedRows(integer r);

   /**
    * Blocks attempts to access this attribute.
    * 
    * @param   p
    *          Not used.
    */
   public void setParent(Object p);

   /**
    * Sets the PARENT attribute to the resource specified in the given handle.
    * 
    * @param   h
    *          The handle with the parent resource.
    */
   public void setParentHandle(handle h);

   /**
    * Sets the input (prompt/edit mode) color.
    * 
    * @param    inputColor
    *           The color to use for when in "input" mode.
    */
   public void setPfcolor(Color inputColor);

   /**
    * Sets the input (prompt/edit mode) color.
    *
    * @param    param
    *           The color to use.
    */
   public void setPfColor(NumberType param);

   /**
    * Blocks attempts to access this attribute.
    * 
    * @param   q
    *          Not used.
    */
   public void setQuery(P2JQuery q);

   /**
    * Sets the READ-ONLY attribute of the widget.
    * 
    * @param    r
    *           <code>true</code> if the widget should be write-protected.
    */
   public void setReadOnly(logical r);

   /**
    * Blocks attempts to access this widget method.
    * 
    * @param   n
    *          Not used.
    * @param   conditional
    *          Not used.
    * 
    * @return  nothing.
    */
   public logical setRepositionedRow(integer n, boolean conditional);

   /**
    * Blocks attempts to access this attribute.
    * 
    * @param   row
    *          Not used.
    */
   public void setRow(double row);

   /**
    * Blocks attempts to access this attribute.
    * 
    * @param   row
    *          Not used.
    */
   public void setRow(NumberType row);

   /**
    * Set the current value in the screen buffer of the backing data for
    * this widget.  If the given value is <code>null</code> then this
    * widget will be set to the uninitialized value.
    *
    * @param    value
    *           The new value for the widget, use <code>null</code> to set
    *           the value as uninitialized.
    */
   public void setScreenValue(character value);

   /**
    * Blocks attempts to access this attribute.
    * 
    * @param   s
    *          Not used.
    */
   public void setScrollable(boolean s);

   /**
    * Blocks attempts to access this attribute.
    * 
    * @param   s
    *          Not used.
    */
   public void setScrollable(logical s);

   /**
    * Blocks attempts to access this attribute.
    * 
    * @param   s
    *          Not used.
    */
   public void setSelected(boolean s);

   /**
    * Blocks attempts to access this attribute.
    * 
    * @param   s
    *          Not used.
    */
   public void setSelected(logical s);

   /**
    * Sets new value of SENSITIVE attribute.
    *
    * @param s
    *        new value of SENSITIVE attribute
    */
   public void setSensitive(logical s);

   /**
    * Sets the TAB-STOP option.
    * 
    * @param    tabStop
    *           <code>true</code> if the option is set to ON.
    */
   public void setTabStop(boolean tabStop);

   /**
    * Blocks attempts to access this attribute.
    * 
    * @param   t
    *          Not used.
    */
   public void setTitle(character t);

   /**
    * Blocks attempts to access this attribute.
    * 
    * @param   t
    *          Not used.
    */
   public void setTitle(String t);

   /**
    * Sets new value of TOOLTIP attribute.
    *
    * @param tooltip
    *        new value of TOOLTIP attribute
    */
   public void setTooltip(character tooltip);

   /**
    * Sets new value of TOOLTIP attribute.
    *
    * @param tooltip
    *        new value of TOOLTIP attribute
    */
   public void setTooltip(String tooltip);

   /**
    * Registers a validation method for any input into this widget.
    * 
    * @param    validator
    *           An instance of <code>Validateable</code> interface to call to
    *           perform input validation.
    */
   public void setValidatable(Validatable validator);

   /**
    * Blocks attempts to access this attribute.
    * 
    * @param   direction
    *          Not used.
    */
   public void setVertical(boolean direction);

   /**
    * Set the VISIBLE writable attribute.
    *
    * @param    visible
    *           The new value for the VISIBLE attribute.
    */
   public void setVisible(boolean visible);

   /**
    * Set the VISIBLE writable attribute.
    *
    * @param    visible
    *           The new value for the VISIBLE attribute.
    */
   public void setVisible(logical visible);

   /**
    * Blocks attempts to access this attribute.
    * 
    * @param   x
    *          Not used.
    */
   public void setX(integer x);

   /**
    * Blocks attempts to access this attribute.
    * 
    * @param   y
    *          Not used.
    */
   public void setY(integer y);

   /**
    * Underlines this widget on the screen.
    * <p>
    * This method corresponds to the Progress statement: UNDERLINE
    */
   public void underline();

   /**
    * Underlines this widget on the screen.
    * <p>
    * This method corresponds to the Progress statement: UNDERLINE
    *
    * @param    stream
    *           An explicit stream which will be used for the output.
    */
   public void underline(Stream stream);

   /**
    * Processes validation rules (if they exist), error checking and any
    * other constraint processing that needs to be applied.  This is used
    * to check if a proposed update to an object will pass validation.
    *
    * @param    val
    *           The new value that is proposed for assignment to the object.
    *
    * @return   <code>null</code> if validation succeeds, error message
    *           otherwise.
    */
   public String validate(BaseDataType val);

   /**
    * Shows this widget.
    * <p>
    * This method corresponds to the Progress statement: VIEW widget
    */
   public void view();

   /**
    * Gets the integer handle value of the window containing the widget.
    * 
    * @return   The integer handle value.
    */
   public integer getHWND();

   /**
    * Blocks attempts to access this attribute.
    * 
    * @return  nothing.
    */
   public handle getFirstTabItem();
   /**
    * Blocks attempts to access this attribute.
    * 
    * @param   h
    *          Not used.
    */
   public void setFirstTabItem(handle h);

   /**
    * Assigns the current widget to the tab position before the specified in the same field group.
    *
    * @param   widget
    *          The handle of the widget before whose we need to make new assignment.
    *
    * @return  <code>true</code> if operation was successful.
    */
   public logical moveBeforeTabItem(GenericWidget widget);

   /**
    * Assigns the current widget to the tab position before the specified in the same field group.
    *
    * @param   widget
    *          The handle of the widget before whose we need to make new assignment.
    *
    * @return  <code>true</code> if operation was successful.
    */
   public logical moveBeforeTabItem(handle widget);
}